@@ -32,6 +32,9 @@ SQL::Translator::Producer::PostgreSQL - PostgreSQL producer for SQL::Translator
 Creates a DDL suitable for PostgreSQL.  Very heavily based on the Oracle
 producer.
 
+Now handles PostGIS Geometry and Geography data types on table definitions.
+Does not yet support PostGIS Views.
+	
 =cut
 
 use strict;
@@ -42,7 +45,7 @@ $DEBUG = 0 unless defined $DEBUG;
 
 use base qw(SQL::Translator::Producer);
 use SQL::Translator::Schema::Constants;
-use SQL::Translator::Utils qw(debug header_comment);
+use SQL::Translator::Utils qw(debug header_comment parse_dbms_version);
 use Data::Dumper;
 
 my ( %translate, %index_name );
@@ -181,7 +184,9 @@ sub produce {
     my $add_drop_table   = $translator->add_drop_table;
     my $schema           = $translator->schema;
     my $pargs            = $translator->producer_args;
-    my $postgres_version = $pargs->{postgres_version} || 0;
+    my $postgres_version = parse_dbms_version(
+        $pargs->{postgres_version}, 'perl'
+    );
 
     my $qt = $translator->quote_table_names ? q{"} : q{};
     my $qf = $translator->quote_field_names ? q{"} : q{};
@@ -190,6 +195,7 @@ sub produce {
     push @output, header_comment unless ($no_comments);
 
     my (@table_defs, @fks);
+    my %type_defs;
     for my $table ( $schema->get_tables ) {
 
         my ($table_def, $fks) = create_table($table, { 
@@ -198,6 +204,7 @@ sub produce {
             no_comments       => $no_comments,
             postgres_version  => $postgres_version,
             add_drop_table    => $add_drop_table,
+            type_defs         => \%type_defs,
         });
 
         push @table_defs, $table_def;
@@ -213,6 +220,7 @@ sub produce {
       });
     }
 
+    push @output, map { "$_;\n\n" } values %type_defs;
     push @output, map { "$_;\n\n" } @table_defs;
     if ( @fks ) {
         push @output, "--\n-- Foreign Key Definitions\n--\n\n" unless $no_comments;
@@ -315,6 +323,12 @@ sub next_unused_name {
     return $name;
 }
 
+sub is_geometry
+{
+	my $field = shift;
+	return 1 if $field->data_type eq 'geometry' || $field->data_type eq 'geography';
+}
+
 sub create_table 
 {
     my ($table, $options) = @_;
@@ -324,6 +338,7 @@ sub create_table
     my $no_comments = $options->{no_comments} || 0;
     my $add_drop_table = $options->{add_drop_table} || 0;
     my $postgres_version = $options->{postgres_version} || 0;
+    my $type_defs = $options->{type_defs} || {};
 
     my $table_name = $table->name or next;
     my ( $fql_tbl_name ) = ( $table_name =~ s/\W(.*)$// ) ? $1 : q{};
@@ -333,7 +348,7 @@ sub create_table
     $table->name($table_name_ur);
 
 # print STDERR "$table_name table_name\n";
-    my ( @comments, @field_defs, @sequence_defs, @constraint_defs, @type_defs, @type_drops, @fks );
+    my ( @comments, @field_defs, @sequence_defs, @constraint_defs, @fks );
 
     push @comments, "--\n-- Table: $table_name_ur\n--\n" unless $no_comments;
 
@@ -353,8 +368,7 @@ sub create_table
                                                  quote_field_names => $qf,
                                                  table_name => $table_name_ur,
                                                  postgres_version => $postgres_version,
-                                                 type_defs => \@type_defs,
-                                                 type_drops => \@type_drops,
+                                                 type_defs => $type_defs,
                                                  constraint_defs => \@constraint_defs,});
     }
 
@@ -399,16 +413,12 @@ sub create_table
     my $create_statement;
     $create_statement = join("\n", @comments);
     if ($add_drop_table) {
-        if ($postgres_version >= 8.2) {
+        if ($postgres_version >= 8.002) {
             $create_statement .= qq[DROP TABLE IF EXISTS $qt$table_name_ur$qt CASCADE;\n];
-            $create_statement .= join (";\n", @type_drops) . ";\n"
-                if $postgres_version >= 8.3 && scalar @type_drops;
         } else {
             $create_statement .= qq[DROP TABLE $qt$table_name_ur$qt CASCADE;\n];
         }
     }
-    $create_statement .= join(";\n", @type_defs) . ";\n"
-        if $postgres_version >= 8.3 && scalar @type_defs;
     $create_statement .= qq[CREATE ${temporary}TABLE $qt$table_name_ur$qt (\n].
                             join( ",\n", map { "  $_" } @field_defs, @constraint_defs ).
                             "\n)"
@@ -417,6 +427,15 @@ sub create_table
     $create_statement .= ( $create_statement =~ /;$/ ? "\n" : q{} )
         . join(";\n", @index_defs);
 
+	#
+	# Geometry
+	#
+	if(grep { is_geometry($_) } $table->get_fields){
+        $create_statement .= ";";
+		$create_statement .= "\n".join("\n", map { drop_geometry_column($_) if is_geometry($_); } $table->get_fields) if $options->{add_drop_table};
+		$create_statement .= "\n".join("\n", map { add_geometry_column($_) if is_geometry($_);} $table->get_fields);
+	}
+
     return $create_statement, \@fks;
 }
 
@@ -468,8 +487,7 @@ sub create_view {
         my $table_name = $field->table->name;
         my $constraint_defs = $options->{constraint_defs} || [];
         my $postgres_version = $options->{postgres_version} || 0;
-        my $type_defs = $options->{type_defs} || [];
-        my $type_drops = $options->{type_drops} || [];
+        my $type_defs = $options->{type_defs} || {};
 
         $field_name_scope{$table_name} ||= {};
         my $field_name    = $field->name;
@@ -491,11 +509,16 @@ sub create_view {
         # todo deal with embedded quotes
         my $commalist = join( ', ', map { qq['$_'] } @$list );
 
-        if ($postgres_version >= 8.3 && $field->data_type eq 'enum') {
-            my $type_name = $field->table->name . '_' . $field->name . '_type';
+        if ($postgres_version >= 8.003 && $field->data_type eq 'enum') {
+            my $type_name = $extra{'custom_type_name'} || $field->table->name . '_' . $field->name . '_type';
             $field_def .= ' '. $type_name;
-            push @$type_defs, "CREATE TYPE $type_name AS ENUM ($commalist)";
-            push @$type_drops, "DROP TYPE IF EXISTS $type_name";
+            my $new_type_def = "DROP TYPE IF EXISTS $type_name CASCADE;\n" .
+                               "CREATE TYPE $type_name AS ENUM ($commalist)";
+            if (! exists $type_defs->{$type_name} ) {
+                $type_defs->{$type_name} = $new_type_def;
+            } elsif ( $type_defs->{$type_name} ne $new_type_def ) {
+                die "Attempted to redefine type name '$type_name' as a different type.\n";
+            }
         } else {
             $field_def .= ' '. convert_datatype($field);
         }
@@ -503,28 +526,68 @@ sub create_view {
         #
         # Default value 
         #
-        my $default = $field->default_value;
-        if ( defined $default ) {
-            SQL::Translator::Producer->_apply_default_value(
-              \$field_def,
-              $default,
-              [
-                'NULL'              => \'NULL',
-                'now()'             => 'now()',
-                'CURRENT_TIMESTAMP' => 'CURRENT_TIMESTAMP',
-              ],
-            );
-        }
+        SQL::Translator::Producer->_apply_default_value(
+          $field,
+          \$field_def,
+          [
+            'NULL'              => \'NULL',
+            'now()'             => 'now()',
+            'CURRENT_TIMESTAMP' => 'CURRENT_TIMESTAMP',
+          ],
+        );
 
         #
         # Not null constraint
         #
         $field_def .= ' NOT NULL' unless $field->is_nullable;
 
+		#
+		# Geometry constraints
+		#
+		if(is_geometry($field)){
+			foreach ( create_geometry_constraints($field) ) {
+				my ($cdefs, $fks) = create_constraint($_, 
+													  { 
+														  quote_field_names => $qf,
+														  quote_table_names => $qt,
+														  table_name => $table_name,
+													  });
+				push @$constraint_defs, @$cdefs;
+				push @$fks, @$fks;
+			}
+        }
+		
         return $field_def;
     }
 }
 
+sub create_geometry_constraints{
+	my $field = shift;
+
+	my @constraints;
+	push @constraints, SQL::Translator::Schema::Constraint->new(
+							name       => "enforce_dims_".$field->name,
+							expression => "(st_ndims($field) = ".$field->{extra}{dimensions}.")",
+							table 	   => $field->table,
+							type       => CHECK_C,
+						);
+						
+	push @constraints, SQL::Translator::Schema::Constraint->new(
+							name       => "enforce_srid_".$field->name,
+							expression => "(st_srid($field) = ".$field->{extra}{srid}.")",
+							table 	   => $field->table,
+							type       => CHECK_C,
+						);
+	push @constraints, SQL::Translator::Schema::Constraint->new(
+							name       => "enforce_geotype_".$field->name,
+							expression => "(geometrytype($field) = '".$field->{extra}{geometry_type}."'::text OR $field IS NULL)",
+							table 	   => $field->table,
+							type       => CHECK_C,
+						);
+						
+	return @constraints;
+}
+
 sub create_index
 {
     my ($index, $options) = @_;
@@ -728,6 +791,11 @@ sub alter_field
         if($from_field->table->name ne $to_field->table->name);
 
     my @out;
+    
+    # drop geometry column and constraints
+	push @out, drop_geometry_column($from_field) if is_geometry($from_field);
+	push @out, drop_geometry_constraints($from_field) if is_geometry($from_field);
+    
     push @out, sprintf('ALTER TABLE %s ALTER COLUMN %s SET NOT NULL',
                        $to_field->table->name,
                        $to_field->name) if(!$to_field->is_nullable and
@@ -779,7 +847,10 @@ sub alter_field
                        $to_field->name)
         if ( !defined $new_default && defined $old_default );
     
-
+	# add geometry column and contraints
+	push @out, add_geometry_column($to_field) if is_geometry($to_field);
+	push @out, add_geometry_constraints($to_field) if is_geometry($to_field);
+	
     return wantarray ? @out : join("\n", @out);
 }
 
@@ -792,6 +863,8 @@ sub add_field
     my $out = sprintf('ALTER TABLE %s ADD COLUMN %s',
                       $new_field->table->name,
                       create_field($new_field));
+    $out .= "\n".add_geometry_column($new_field) if is_geometry($new_field);
+    $out .= "\n".add_geometry_constraints($new_field) if is_geometry($new_field);
     return $out;
 
 }
@@ -803,16 +876,62 @@ sub drop_field
     my $out = sprintf('ALTER TABLE %s DROP COLUMN %s',
                       $old_field->table->name,
                       $old_field->name);
-
+	$out .= "\n".drop_geometry_column($old_field) if is_geometry($old_field);
     return $out;    
 }
 
+sub add_geometry_column{
+	my ($field,$options) = @_;
+	
+	my $out = sprintf("INSERT INTO geometry_columns VALUES ('%s','%s','%s','%s','%s','%s','%s')",
+						'',
+						$field->table->schema->name,
+						$options->{table} ? $options->{table} : $field->table->name,
+						$field->name,
+						$field->{extra}{dimensions},
+						$field->{extra}{srid},
+						$field->{extra}{geometry_type});
+    return $out;
+}
+
+sub drop_geometry_column
+{
+	my $field = shift;
+	
+	my $out = sprintf("DELETE FROM geometry_columns WHERE f_table_schema = '%s' AND f_table_name = '%s' AND f_geometry_column = '%s'",
+						$field->table->schema->name,
+						$field->table->name,
+						$field->name);
+    return $out;
+}
+
+sub add_geometry_constraints{
+	my $field = shift;
+	
+	my @constraints = create_geometry_constraints($field);
+
+	my $out = join("\n", map { alter_create_constraint($_); } @constraints);
+	
+	return $out;
+}
+
+sub drop_geometry_constraints{
+	my $field = shift;
+	
+	my @constraints = create_geometry_constraints($field);
+	
+	my $out = join("\n", map { alter_drop_constraint($_); } @constraints);
+	
+	return $out;
+}
+
 sub alter_table {
     my ($to_table, $options) = @_;
     my $qt = $options->{quote_table_names} || '';
     my $out = sprintf('ALTER TABLE %s %s',
                       $qt . $to_table->name . $qt,
                       $options->{alter_table_action});
+    $out .= "\n".$options->{geometry_changes} if $options->{geometry_changes};
     return $out;
 }
 
@@ -820,6 +939,13 @@ sub rename_table {
     my ($old_table, $new_table, $options) = @_;
     my $qt = $options->{quote_table_names} || '';
     $options->{alter_table_action} = "RENAME TO $qt$new_table$qt";
+
+	my @geometry_changes;
+	push @geometry_changes, map { drop_geometry_column($_); } grep { is_geometry($_) } $old_table->get_fields;
+	push @geometry_changes, map { add_geometry_column($_, { table => $new_table }); } grep { is_geometry($_) } $old_table->get_fields;
+	
+    $options->{geometry_changes} = join ("\n",@geometry_changes) if scalar(@geometry_changes);
+    
     return alter_table($old_table, $options);
 }
 
@@ -874,7 +1000,12 @@ sub alter_create_constraint {
 sub drop_table {
     my ($table, $options) = @_;
     my $qt = $options->{quote_table_names} || '';
-    return "DROP TABLE $qt$table$qt CASCADE";
+    my $out = "DROP TABLE $qt$table$qt CASCADE";
+    
+    my @geometry_drops = map { drop_geometry_column($_); } grep { is_geometry($_) } $table->get_fields;
+
+    $out .= "\n".join("\n",@geometry_drops) if scalar(@geometry_drops);
+    return $out;
 }
 
 1;
